##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
#
# Copyright (c) 2000 Andy McKay
# All rights reserved.
#
# Distributed under the Zope Public License (above)
#
##############################################################################
#
# No result and IP Filtering added by Aleksander Salwa, ololo@webcraft.pl
# Added April 14, 2001
#
##############################################################################
#
# More changes and additions - see changes.txt, chris@d-designz.co.nz
# Added May 15, 2002
#
##############################################################################

__version__ = '1.3.3'

from Globals import HTMLFile
from Globals import MessageDialog
from Globals import Persistent

import OFS.SimpleItem
import Acquisition
import AccessControl.Role

import os
import string
import Globals # so I can get Globals data_dir
import time

manage_addFSCounterForm = HTMLFile('FSCounterAdd', globals())

def manage_addFSCounter(self, id, title='', REQUEST=None):
    ''' Adds a new object '''
    self._setObject(id, FSCounter(id, title))
    if REQUEST is not None:
            return self.manage_main(self, REQUEST)

class FSCounter(OFS.SimpleItem.Item, Persistent, Acquisition.Implicit, AccessControl.Role.RoleManager):
    meta_type = 'FSCounter'

    manage_main = HTMLFile('FSCounterEdit', globals())
    manage_help = HTMLFile('FSCounterHelp', globals())

    manage_options = (
        {'label': 'Edit', 'action': 'manage_main'},
        {'label': 'View', 'action': ''},
        {'label': 'Security', 'action': 'manage_access'},
        {'label': 'Help', 'action': 'manage_help'},
            )

    __ac_permissions__=(
        ('View management screens', ('manage_tabs','manage_edit','reset','manage_help')),
        ('Change permissions', ('manage_access',) ),
        ('View', ('', 'index_html',) ),
        )

    def __init__(self, id, title ='', initialvalue=''):
        """initialise a new instance of FSCounter"""
        self.id = id
        self.title = title

        # choose a unique file name
        self.filename = '%s.%s.counter' % (self.id, str(time.time()))

        # use cookie
        self.cookie_yn = 0
        self.cookie_name = self.filename

        # use server-side unique ip tracking
        self.ip_tracking = 0

        # number of zero's to append eg: 000001 = 5
        self.padded_value = 5
        self.default_value = 'get_padded_images'

        # images folder
        self.images_folder = 'misc_/FSCounter'

        # addresses to ignore (no counter increase):
        # (tuple of (address, mask) tuples)
        # addresses and masks are stored as (long) integers - for better performance
        self.ignore_addrs = ()

        # make sure we reset
        if initialvalue == '':
            self.reset()
        else:
            self.reset(initialvalue=initialvalue)

    def manage_beforeDelete(self, item, container):
        ''' clean up '''
        self.__delete_()

    def manage_edit(self, title, cookie_yn=0, cookie_name='', padded_value='5', images_folder='misc_/FSCounter', image_type='', ignore_addrs=(), newvalue='', ip_tracking=0, REQUEST=None):
        """edits the object"""
        self.title = title

        # will be there
        self.default_value = str(REQUEST['default_value'])

        self.padded_value = padded_value
        self.cookie_yn = cookie_yn
        self.cookie_name = cookie_name

        self.ip_tracking = ip_tracking

        # hmmm crappy default
        if self.cookie_name == '': self.cookie_name = self.id
        self.images_folder = images_folder
        self.image_type = image_type

        ignAdTmp=[]
        for addr_range in ignore_addrs:
            if addr_range=='':
                continue
            adr=string.split(addr_range, ':')[0]
            msk=string.split(addr_range, ':')[1]
            adrParts=string.split(adr, '.')
            mskParts=string.split(msk, '.')
            adrInt=(long(adrParts[0])<<24)+(int(adrParts[1])<<16)+(int(adrParts[2])<<8)+int(adrParts[3])
            mskInt=(long(mskParts[0])<<24)+(int(mskParts[1])<<16)+(int(mskParts[2])<<8)+int(mskParts[3])
            ignAdTmp.append((adrInt, mskInt))
        self.ignore_addrs=tuple(ignAdTmp)

        if newvalue != '':
            self.__write_(newvalue)

        if REQUEST is not None:
            return MessageDialog(title = 'Edited', message = "Editing successful", action = './manage_main',)

    def __write_(self, newvalue=''):
        ''' write newvalue to file or increment '''
        value = self.read()

        if newvalue == '':
            value = value + 1
        else:
            value = int(newvalue)
        try:
            outfile = open(str(Globals.data_dir) + os.sep + str(self.filename), 'wb')
            outfile.write(str(value))
            outfile.close()
        except:
            pass
        return int(value)

    def read(self):
        ''' read file '''
        try:
            infile = open(str(Globals.data_dir) + os.sep + str(self.filename), 'rb')
            readfile = infile.read()
            if readfile == '':         # check for blank
                value = 0              #    and if so, set to 0
            else:
                value = int(readfile)
            infile.close()
        except IOError:
            # error creating file
            value = 0

        return value

    def __delete_(self):
        ''' delete the object '''
        try:
            os.unlink(Globals.data_dir + os.sep + self.filename)
            os.unlink(Globals.data_dir + os.sep + self.filename + '.ip')
        except:
            pass # error deleting, oh well

    def reset(self, REQUEST=None, initialvalue=''):
        ''' reset cookie '''
        initialvalue = getattr(REQUEST,'initialvalue','0')
        self.__write_(initialvalue)
        if REQUEST is not None:
            return MessageDialog(title = 'Reset', message = 'Counter set to %s' % str(initialvalue), action = './manage_main',)

    def get_value(self, value=None):
        ''' returns a value '''
        if value is None:
            value = self.read()
        return value

    def get_padded_value(self, value=None):
        ''' returns a padded value '''
        value = self.get_value(value=value)
        return ('%0'+str(self.padded_value)+'d') % value

    def get_null(self, value=None):
        return ' '

    def get_images(self, value=None):
        ''' returns images '''
        output  = ''
        value = self.get_value(value=value)
        for num in str(value):
            output = output + self.get_each_image(num)

        return output
 
    def pic(self, position, RESPONSE=None, REQUEST={}):
        ''' returns the actual image at the given position '''
        if int(position) == 1:     # increment
            self.increment(REQUEST=REQUEST)
        value = str(self.get_value())
        lenvalue = len(value)
        position = lenvalue - int(position)
        if position < 0 or position >= lenvalue:
            num = '0'
        else:
            num = value[position]
        if RESPONSE is not None:
            RESPONSE.redirect(self.image_filename(num))
        return self.image_filename(num)

    def image_filename(self, num):
        if self.images_folder == 'misc_/FSCounter':
            image_type = getattr(self, 'image_type', '')
        else:
            image_type = ''
        return '%s/%s%s.gif' % (self.images_folder, image_type, num)
        
    def get_each_image(self, num):
        ''' gets each image '''
        st = '<img src="%s" alt="%s" title="">' % (self.image_filename(num), num)
        return st

    def get_padded_images(self,value=None):
        ''' returns images '''
        return self.get_images(value=self.get_padded_value(value=None))

    def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):
        ''' standard_view '''
        method = getattr(self, self.default_value)
        return method(value = self.increment(REQUEST=REQUEST))

    def increment(self, REQUEST):
        ''' increment (if necessary) '''
        if self.ignore_ip(REQUEST=REQUEST):
            return self.read()
        if hasattr(self,'ip_tracking') and self.ip_tracking:
            found = 0
            rewrite = 1
            ip = '%s\n' % (self.REQUEST.get('HTTP_X_FORWARDED_FOR', '') or self.REQUEST.get('REMOTE_ADDR', ''))
            try:
                infile = open(str(Globals.data_dir) + os.sep + str(self.filename) + '.ip', 'r')
                lines = infile.readlines()
                infile.close()
                firstline = 1
                for line in lines:
                    if firstline:
                        if line != time.strftime('%Y-%m-%d') + '\n':
                            break
                        else:
                            rewrite = 0
                        firstline = 0
                    else:
                        if line == ip:
                            found = 1
                            break
            except:
                pass
            if rewrite:
                try:
                    outfile = open(str(Globals.data_dir) + os.sep + str(self.filename) + '.ip', 'w')
                    outfile.write(time.strftime('%Y-%m-%d')+'\n')
                    outfile.write(ip)
                    outfile.close()
                except:
                    pass
            elif not found:
                try:
                    outfile = open(str(Globals.data_dir) + os.sep + str(self.filename) + '.ip', 'a')
                    outfile.write(ip)
                    outfile.close()
                except:
                    pass
            #return lines
            if found:
                return self.read()
        elif self.cookie_yn:
            if self.cookie_yn and self.REQUEST.has_key(self.cookie_name):
                return self.read()
            else:
                self.REQUEST.RESPONSE.setCookie(self.cookie_name, 1)
        return self.__write_()

    def ignore_ip(self, REQUEST):
        ''' check if the ip should be ignored '''
        ignore = 0
        if self.ignore_addrs:
            rAdrSplit=string.split(self.REQUEST.get('HTTP_X_FORWARDED_FOR', '') or self.REQUEST.get('REMOTE_ADDR', ''), '.')
            rAdrInt=(int(rAdrSplit[0])<<24)+(int(rAdrSplit[1])<<16)+(int(rAdrSplit[2])<<8)+int(rAdrSplit[3])
            for addr_range in self.ignore_addrs:
                if rAdrInt&addr_range[1] == addr_range[0]:
                    ignore = 1
                    break
        return ignore

#    index_html = __call__
    def index_html(self, client=None, REQUEST={}, RESPONSE=None, **kw):
        ''' default html page '''
        return '<html><body>%s</body></html>' % self.__call__(client, REQUEST, RESPONSE, **kw)
        
